import json
import aiohttp
from quaero.utils import get_api_key
from colorama import Style
import time

# Fn to call the bot and print its response as it streams in
async def fetch_data(conversation, fetched_data_flag):
    api_key = get_api_key()
    headers = {"Authorization": f"Bearer {api_key}", "content-type": "application/json"}
    url = f"https://api.openai.com/v1/chat/completions"
    payload = {
        "model": "gpt-3.5-turbo",
        "messages": [value for value in conversation.values()],
        "temperature": 0.7,
        "stream": True,
    }

    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(
                url, data=json.dumps(payload), headers=headers
            ) as response:
                if response.status == 200:
                    # Response received - can stop printing the 'Thinking...' message
                    fetched_data_flag.set()
                    timestamp = time.time()
                    whole_response = []

                    print(Style.RESET_ALL, "\n\n> 🤖")
                    # The streaming response is SSEs - process each event by iterating through the chunks,
                    # instead of waiting for the whole request to finish
                    # https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events
                    async for chunk in response.content:
                        try:
                            parsed = json.loads(chunk.decode("utf-8").removeprefix("data: "))
                            # TODO: validate that the event looks like what you expect, and handle better if not
                            chunk_message = parsed["choices"][0]["delta"]["content"]
                            # TODO: just have a string that gets longer?
                            # Append each chunk to a list that will be added to the conversation object
                            whole_response.append(chunk_message)
                            print(chunk_message, end="", flush=True)
                        except:
                            pass
                    conversation[f"{timestamp}-bot"] = {'content': "".join(whole_response), 'role': 'assistant'}
                else:
                    print(f"Error calling OpenAI.  API response: {response.status}")
    except aiohttp.ClientError as e:
        print(f"An error occured: {e}")


#!/usr/bin/env python3

import argparse
import os
import asyncio
import signal
import sys
from colorama import Fore, Style
import time
import json

from quaero.openai import fetch_data
from quaero.utils import print_waiting, save_to_file, check_for_config
from quaero.consts import app_directory, api_key_key, config_path

async def main_impl():
    conversation = {}
    
    # Handler for ?? (ctrl + c)
    def quit_handler(sig, frame):
        print("\nExiting script...")
        id = save_to_file(conversation)
        if id:
            print("Conversation ID: ", id)
        sys.exit(0)

    signal.signal(signal.SIGINT, quit_handler)

    parser = argparse.ArgumentParser(description="🤖 ChatGPT CLI")
    parser.add_argument(
        "command", choices=["init"], help="Set up your OpenAI credentials", nargs="?"
    )
    parser.add_argument(
        "-c", "--conversation", help="Use with the ID printed at the end of a previous conversation to resume that same conversation.  Will not work if the conversation has been deleted.", nargs=1
    )
    args = parser.parse_args()

    # Handle init setup
    if args.command == "init":
        print(
            Fore.YELLOW + "Welcome to quaero! To get started you will need to generate an OpenAI API key and paste it here to save it to your quaero config.\n\nVisit " + Fore.WHITE + "https://platform.openai.com/api-keys." + Fore.YELLOW + "\n\nThe key should be like sk-{20chars}\n" + Style.RESET_ALL
        )
        key = input("Please paste your key: ")
        os.makedirs(app_directory, exist_ok=True)
        with open(config_path, "w+") as file:
            file.write(f"{api_key_key}={key.strip()}")
        print("Success!  You are ready to ask your first question.")

    # Make sure an API key is set
    try:
        check_for_config()
    except Exception as e:
        print(Fore.RED, e, Style.RESET_ALL)
        sys.exit(1)

    # TODO: clean out old conversations
        
    if args.conversation:
        d, t = args.conversation[0].split('/')
        conversation_path = os.path.join(app_directory, d, t)
        if not os.path.isfile(conversation_path):
            print(Fore.RED + "That conversation ID was not found" + Style.RESET_ALL)
            sys.exit(1)
        with open(conversation_path) as file:
            conversation = json.load(file)

    while True:
        fetched_data_flag = asyncio.Event()

        # Prompt user for their question
        user_input = input("> ")

        # Store their question in the conversation dictionary
        conversation[time.time()] = {'content': user_input, 'role': 'user'}

        # Simultaneous kick off the API request and printing the waiting message
        # The fetch_data function also writes the response to the conversation dictionary
        await asyncio.gather(
            fetch_data(conversation, fetched_data_flag), print_waiting(fetched_data_flag)
        )

        print("\n")


def main():
    # Asyncio lib needed so the data request and wait message can happen simultaneously (requests lib is blocking)
    asyncio.run(main_impl())


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="quaero",
    version="1.1.2",
    description="Simple CLI tool for asking ChatGPT questions in the CLI",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Gillian Yeomans",
    author_email="hello@gillian.codes",
    url="https://github.com/yeo-yeo/chatgpt-cli-python",
    packages=find_packages(),
    install_requires=["aiohttp", "colorama", "python-dotenv"],
    entry_points={
        "console_scripts": [
            "quaero=quaero.index:main",
        ],
    },
)



from dotenv import dotenv_values, find_dotenv
import os
from colorama import Fore, Style
import asyncio
import datetime
import json

from quaero.consts import config_path, api_key_key, app_directory

def check_for_config():
    # TODO: specifically check for existence of file rather than keys?
    config = get_config()
    if not config.get("API_KEY", None):
        raise Exception("Config not set up properly.  Have you run `quaero init`?")


def get_config():
    return dotenv_values(config_path)


def get_api_key():
    config = get_config()
    api_key = config.get(api_key_key, None)
    if api_key == None:
        print(Fore.RED + "Unable to find API key.  Have you created run `quaero init`?" + Style.RESET_ALL)
        return
    return api_key


# Print message while waiting for data to be returned, to show that something is happening
async def print_waiting(fetched_data):
    print(Fore.YELLOW + "\nThinking...", end="")
    while not fetched_data.is_set():
        print(".", end="", flush=True)
        await asyncio.sleep(0.5)
    print(Style.RESET_ALL)

# Save conversation so it can be resumed or revisited
def save_to_file(output):
    if len(output.keys()):
        current_datetime = datetime.datetime.now()
        date, time = current_datetime.strftime("%Y%m%d"), current_datetime.strftime(
            "%H%M%S"
        )
        output_directory = os.path.join(app_directory, date)
        os.makedirs(output_directory, exist_ok=True)
        file_path = os.path.join(output_directory, time)
        with open(file_path, "w+") as file:
            file.write(json.dumps(output))
        return f"{date}/{time}"
